
import os
import os.path
from xml.etree import ElementTree as ET
import re
import my_utils
import env
import pack_exception


def modifyFile(filePath, old, new):  
    file_object_read = open(filePath, 'r')   
    try:  
        stringsave=""  
        stringread=file_object_read.readline()  
        while stringread:  
            stringread=stringread.replace(old, new)  
            stringsave=stringsave+stringread  
            stringread=file_object_read.readline()  
        file_object_save = open(filePath, 'w')  
        file_object_save.write(stringsave)  
    finally:  
        file_object_read.close()  
        file_object_save.close()
            
            
def dexTrans2Smali(dexFile, targetDir, baksmali = env.TOOL_DEF_BAKSMALI_JARFILE):
    '''
    @brief 把dex文件反编译成smali格式文件组成的内容
    @param[in] dexFile dex文件路径
    @param[in] targetDir smali文件存放
    @param[in] baksmali反编译需要的辅助jar包
    @return >0:失败; 0:成功
    '''
    dexFile = my_utils.getFullPath(dexFile)
    baksmaliFile = my_utils.getToolPath(baksmali)
    targetDir = my_utils.getFullPath(targetDir)
    
    if not os.path.exists(targetDir):
        os.mkdir(targetDir)    
        
    if not os.path.exists(dexFile) or not os.path.exists(baksmaliFile):
        my_utils.printf("dexTrans2Smali: file %s or %s not exist." % (baksmali,dexFile))
        return 1            
    
    cmd = '%s -jar "%s" -o "%s" "%s"' % (my_utils.getJavaPath('java'),
     baksmaliFile,
     targetDir,
     dexFile)
    ret = my_utils.execFormatCmd(cmd)
    if ret:
        #my_utils.printf("dexTrans2Smali: execFormatCmd(%s) failure ret(%d)." % (cmd,ret))
        return ret
    else:
        return 0
            
def produceNewRFile(packName, decompileFullDir, androidManiFest = 'AndroidManifest.xml'):
    '''生成R文件'''
    fullPath = decompileFullDir
    tempPath = os.path.dirname(decompileFullDir)
    tempPath = tempPath + '/tempRFile'
    print('tempPath=', tempPath)
    if os.path.exists(tempPath):
        my_utils.delete_file_folder(tempPath)
    os.makedirs(tempPath)
    resPath = os.path.join(decompileFullDir, 'res')
    targetResPath = os.path.join(tempPath, 'res')
    my_utils.copyFiles(resPath, targetResPath)
    genPath = os.path.join(tempPath, 'gen')
    print('genPath=', genPath)
    if not os.path.exists(genPath):
        os.mkdir(genPath)
    androidPath = my_utils.getToolPath('android.jar')
    srcManifest = os.path.join(fullPath, androidManiFest)
    aaptPath = my_utils.getToolPath('aapt')
    cmd = '%s p -f -m -J "%s" -S "%s" -I "%s" -M "%s"' % (aaptPath,
     genPath,
     targetResPath,
     androidPath,
     srcManifest)
    ret = my_utils.execFormatCmd(cmd)
    if ret:
        #my_utils.printf("produceNewRFile: execFormatCmd(%s) failure ret=%d" % (cmd,ret))
        return 1
    RPath = packName.replace('.', '/')
    RPath = os.path.join(genPath, RPath)
    RFile = os.path.join(RPath, 'R.java')
    print('RFile=', RFile)

    #拷贝生成的R文件到指定包名下
    target_RPath = os.path.join(genPath, 'com/facebook')
    if not os.path.exists(target_RPath):
        os.makedirs(target_RPath)
    print('target_RPath=', target_RPath)
    target_RFile = os.path.join(target_RPath, 'R.java')
   
    print('copy RFile=', RFile, ' to target_RFile=', target_RFile)
    ret = my_utils.copyFiles(RFile, target_RFile)
    if ret:
        raise PackException(pack_exception.COPY_RESOURCES_EXCEPTION, 'copy file failed')
    print('remodify script copy R.java success')
    #修改R文件包名
    modifyFile(target_RFile, packName, 'com.facebook')
    print('remodify script modify r file success')
    
    #重新生成dex
    cmd = '"%s" -source %s -target %s -encoding UTF-8 "%s"' % \
        (my_utils.getJavaPath('javac'),env.JAVAC_COMPILE_VERSION,env.JAVAC_COMPILE_VERSION,RFile)
    ret = my_utils.execFormatCmd(cmd)
    if ret:
        #my_utils.printf("produceNewRFile: execFormatCmd(%s) failure ret=%d" % (cmd,ret))
        return 2
    
    cmd = '"%s" -source %s -target %s -encoding UTF-8 "%s"' % \
        (my_utils.getJavaPath('javac'),env.JAVAC_COMPILE_VERSION,env.JAVAC_COMPILE_VERSION,target_RFile)
    ret = my_utils.execFormatCmd(cmd)
    if ret:
        #my_utils.printf("produceNewRFile: execFormatCmd(%s) failure ret=%d" % (cmd,ret))
        return 2
    dexPath = os.path.join(tempPath, 'class.dex')
    #if platform.system() == 'Windows':
        #dxTool = my_utils.getToolPath('dx.bat')
        #cmd = '"%s" --dex --output="%s" "%s"' % (dxTool, dexPath, genPath)
    #else:
    dxTool = my_utils.getToolPath('dx.jar')
    cmd = '"%s" -jar "%s" --dex --output="%s" "%s"' % \
        (my_utils.getJavaPath('java'),dxTool, dexPath, genPath)
    ret = my_utils.execFormatCmd(cmd)
    if ret:
        #my_utils.printf("produceNewRFile: execFormatCmd(%s) failure ret=%d" % (cmd,ret))
        return 3
    smaliPath = os.path.join(fullPath, 'smali')
    ret = dexTrans2Smali(dexPath, smaliPath, 'baksmali-1.4.1.jar')
    if ret:
        return 4
    else:
        return 0
            

def script(decompileDir, sdkpath, targetPackageName, sdk):
    ''' 
    @brief 为sdk做特殊处理调度，做特殊处理的sdk需要提供script.pyc类似的脚本文件
    @param decompileDir: 反编译目录
    @param targetPackageName 要生成的包的包名
    @param decompileDir 反编译目录
    @param sdk php传递的sdk参数
    '''
    print('facebook remodify invoking');
    print('facebook remodify script produce R file')
    ret = produceNewRFile(targetPackageName, decompileDir);
    print('facebook remodify script produce R file ret=', ret)
    if ret:
        raise PackException(pack_exception.PRODUCE_NEW_R_FILE_FAILED, 'produce new r file failed')
    print('facebook remodify script invoked success')
    return 0

# Test Case    
# if __name__ == '__main__':
#     decompileDir = r'E:\svn_online\GodSDK\trunk\GodSDKLetvSDK\GodSDKLetvPay'
#     sdkpath = ''
#     targetPackageName = 'com.boyaa.engineddz.lenovo'
#     sdk = {'Params': {'LetvSDKAppKey':'11', 'LetvSDKSecretKey':'22', 'LetvSDKCallbackUrl':'33', 
#     'StvSDKAppKey':'44','StvSDKSecretKey':'55', 'StvSDKCallbackUrl':'96'}}
#     script(decompileDir, sdkpath, targetPackageName, sdk)

